# -*- coding: utf-8 -*-
# Copyright (c) 2017-2023 Richard Hull and contributors
# See LICENSE.rst for details.

"""
Display drivers for LED Matrices & 7-segment displays (MAX7219) and
RGB NeoPixels (WS2812 / APA102).
"""

__version__ = "1.9.0"
